/* File:      dataset.h
   Date:      6/22/02  (written fall 2001)
   Author:    C. Paciorek
   Purpose:   header file for dataset.C
   Compile:   
   Usage:     
   Arguments: 
   Requires:  
   Reference: 
*/
class sparse_cov_mat_nonstat;
class sparse_cov_mat_nonstat_basis;
class sparse_cov_mat_fullstat;
class sparse_cov_mat_fixed;
class locn_info;

extern locn_info* locations;       /* locations is global because dataset and parms need access to permutation information */

extern int prior;

class dataset {
public:
  dataset(string in_fn, string years_fn_in, int num_years_in,int num_gridpoints_in,double* ints_in, double* slopes_in,double* variances_in,double* delta_in,double* nu_in,double* kappa1_in,double* kappa2_in,double** eigenval1_in,double** eigenval2_in, double** eigenvec1_in, double** eigenvec2_in,string datacorr_fn_in);
  ~dataset(void);
  void update(int accepted,int which);  /* update likelihood values */
  double evaluate_loglik_diff(int which,int first);  /* calc log-likelihood ratio */
  double evaluate_logpost(void);
  void datacov_change(int which,int first);
  double* get_lnz_C(void);
  double* get_diag_C(void);
  double* get_lnz_R(void);
  double* get_diag_R(void);

  double loglik;    /* chain needs access */
  double *sum_data;        /* sum_{t} Y_{l,t}, t=-25,...,25 */
  double *sum_data_by_t;   /* sum_{t} t*Y_{l,t}, t=-25,...,25  */
  sparse_cov_mat_fullstat* stat_cov;      /* stationary covariance of data */
  sparse_cov_mat_nonstat_basis* nonstat_cov;      /* nonstationary covariance of data */
  /*
  sparse_cov_mat_nonstat* nonstat_cov; */     /* nonstationary covariance of data */
  sparse_cov_mat_fixed* fixed_cov;      /* fixed correlation of data */
  int sum_t_squ;                   /* sum_{t} t*t, t=-25,...,25  */
  int num_years;
  int num_test_years;
  int num_train_years;
  double logpost_det;
  double* logpost_test;
  double* SSE_test;
  double* SSSE_test;
  double* etahat;
  double* delta;
private:
  int num_gridpoints;                        /* number of locations */
  mat_pp<double>* values;       /* data stored as each row a location */
  double* ints;             /* intercept parameters */
  double* slopes;           /* slope parameters */
  double* variances;        /* variance parameters, for use in SSSE */
  int* years;

  /* various terms in loglikelihood: */
  double loglik_gamma_stored;
  double loglik_alpha_stored;
  double loglik_beta_stored;
  double loglik_gamma; 
  double loglik_alpha;
  double loglik_beta;

  double* tmpvec1;
  double* tmpvec2;
  double* tmpvec3;
};









